@@ -5,6 +5,8 @@ import random as r
 
 from .logic import Logical
 
+__all__ = ['Interval','I']
+
 class Interval():
     """
     Interval
@@ -12,26 +14,22 @@ class Interval():
     An interval is an uncertain number for which only the endpoints are known, for example if :math:`x=[a,b]`
     then this can be interpreted as :math:`x` being between :math:`a` and :math:`b` but with no more information about the value of :math:`x`
     .
-
     Intervals can be created using::
-
         pba.I(left,right)
         pba.Interval(left,right)
-
     Parameters
     ----------
     left : numeric
         Left side of interval
     right : numeric
         Right side of interval
-
     Attributes
     ----------
     Left : numeric
         Left side of interval
     Right : numeric
         Right side of interval
-    
+
     """
     def __init__(self,Left = None, Right = None):
 
@@ -417,6 +415,96 @@ class Interval():
     lo = left
     hi = right
 
+  def mean(*args):
+        LSum = 0
+        USum = 0
+        DataLen = len(args)
+        for x in args:
+            if x.__class__.__name__ in ("int","float"):
+                x = Interval(x)
+            if x.__class__.__name__ in ("list","tuple"):
+                DataLen = DataLen + (len(x) - 1)
+                for y in x:
+                    if y.__class__.__name__ in ("int","float"):
+                        y = Interval(y)
+                    LSum = LSum + y.Left
+                    USum = USum + y.Right
+            if x.__class__.__name__ == "Interval":
+                LSum = LSum + x.Left
+                USum = USum + x.Right
+            LMean = LSum / DataLen
+            UMean = USum / DataLen
+        return Interval(LMean, UMean)
+
+    def median(*args):
+        LBounds = []
+        LSorted = []
+        UBounds = []
+        USorted = []
+
+        for x in [*args]:
+            if x.__class__.__name__ in ("int","float"):
+                x = Interval(x)
+                LBounds.append(x.Left)
+                UBounds.append(x.Right)
+            if x.__class__.__name__ in ("list","tuple"):
+                for y in x:
+                    if y.__class__.__name__ in ("int","float"):
+                        y = Interval(y)
+                    LBounds.append(y.Left)
+                    UBounds.append(y.Right)
+            if x.__class__.__name__ == "Interval":
+                LBounds.append(x.Left)
+                UBounds.append(x.Right)
+        while (len(LBounds) > 0):
+            MinL = min(LBounds)
+            LSorted.append(MinL)
+            LBounds.remove(MinL)
+        while (len(UBounds) > 0):
+            MinU = min(UBounds)
+            USorted.append(MinU)
+            UBounds.remove(MinU)
+
+        if (len(LSorted) % 2) != 0:
+            LMedian = LSorted[len(LSorted)//2]
+            UMedian = USorted[len(USorted)//2]
+        else:
+            LMedian = (LSorted[len(LSorted)//2] + LSorted[(len(LSorted)//2)-1])/2
+            UMedian = (USorted[len(USorted)//2] + USorted[(len(USorted)//2)-1])/2
+        return Interval(LMedian,UMedian)
+
+    def variance(*args):
+        dataMean = Interval.mean(*args)
+        LBounds = []
+        UBounds = []
+        LDev = []
+        UDev = []
+        DataLen = len(args)
+        for x in [*args]:
+            if x.__class__.__name__ in ("int","float"):
+                x = Interval(x)
+                LBounds.append(x.Left)
+                UBounds.append(x.Right)
+            if x.__class__.__name__ in ("list","tuple"):
+                DataLen = DataLen + (len(x) - 1)
+                for y in x:
+                    if y.__class__.__name__ in ("int","float"):
+                        y = Interval(y)
+                    LBounds.append(y.Left)
+                    UBounds.append(y.Right)
+
+        for y in LBounds:
+            LDev.append(abs(y - dataMean.Left)**2)
+        for z in UBounds:
+            UDev.append(abs(z - dataMean.Right)**2)
+
+        LSDev = (sum(LDev))/DataLen
+        USDev = (sum(UDev))/DataLen
+        return Interval(LSDev, USDev)
+
+    def mode(*args):
+        NotImplemented
+        
     def straddles(self,N, endpoints = True):
         """
         Parameters
@@ -425,14 +513,13 @@ class Interval():
             Number to check
         endpoints : bool
             Whether to include the endpoints within the check
-        
+
         Returns
         -------
         True
             If :math:`\\mathrm{left} \\leq N \\leq \mathrm{right}` (Assuming `endpoints=True`)
         False
             Otherwise
-
         """
         if endpoints:
             if self.Left <= N and self.Right >= N:
@@ -446,15 +533,14 @@ class Interval():
     def straddles_zero(self,endpoints = True):
         """
         Checks whether :math:`0` is within the interval
-        """        
+        """
         return self.straddles(0,endpoints)
 
     def recip(self):
         """
-        Calculates the reciprocle of the interval. 
-
+        Calculates the reciprocle of the interval.
         If :math:`0 \\in [a,b]` it returns a division by zero error
-        
+
         """
         if self.straddles_zero():
             # Cant divide by zero
